@@ -1,5 +1,7 @@
 use tokio::io::{AsyncRead, AsyncWrite};
 
+use anyhow::Result;
+
 use crate::client::inner::Inner;
 use crate::proto::{
     bytes::Bytes,
@@ -7,10 +9,11 @@ use crate::proto::{
 };
 
 pub trait Client: AsyncRead + AsyncWrite + Unpin + Sync + Send + 'static {}
+impl<C> Client for C where C: Send + Sync + Unpin + AsyncWrite + AsyncRead + 'static {}
 
 #[async_trait]
 pub trait AuthMethod {
-    async fn perform_auth<C>(&self, inner: &mut Inner<C>)
+    async fn perform_auth<C>(&self, inner: &mut Inner<C>) -> Result<()>
     where
         C: Client;
 }
@@ -22,15 +25,18 @@ pub struct Login {
 
 #[async_trait]
 impl AuthMethod for Login {
-    async fn perform_auth<C>(&self, inner: &mut Inner<C>)
+    async fn perform_auth<C>(&self, inner: &mut Inner<C>) -> Result<()>
     where
         C: Client,
     {
         let command = Command::Login(CommandLogin {
-            username: Bytes::from(self.username.clone()),
+            userid: Bytes::from(self.username.clone()),
             password: Bytes::from(self.password.clone()),
         });
 
-        let _result = inner.execute(command).await;
+        let result = inner.execute(command).await?;
+        info!("result: {:?}", result.wait().await?);
+
+        Ok(())
     }
 }
