@@ -3,21 +3,20 @@ use std::sync::atomic::{AtomicU32, Ordering};
 use anyhow::Result;
 use futures::{
     future::{self, FutureExt, TryFutureExt},
-    sink::SinkExt,
     stream::StreamExt,
 };
 use tokio::{
-    io::{split, AsyncRead, AsyncWrite, ReadHalf, WriteHalf},
+    io::{split, AsyncRead, AsyncWrite, AsyncWriteExt, BufWriter, ReadHalf, WriteHalf},
     sync::{mpsc, oneshot},
     task::JoinHandle,
 };
 use tokio_rustls::client::TlsStream;
-use tokio_util::codec::{FramedRead, FramedWrite};
+use tokio_util::codec::FramedRead;
 
 use crate::proto::{
     bytes::Bytes,
     command::Command,
-    response::{Response, Tag},
+    response::{Condition, Response, Status, Tag},
     rfc3501::capability as parse_capability,
 };
 
@@ -135,9 +134,13 @@ where
         Ok(false)
     }
 
-    pub async fn upgrade(self) -> Result<Inner<TlsStream<C>>> {
+    pub async fn upgrade(mut self) -> Result<Inner<TlsStream<C>>> {
+        debug!("preparing to upgrade using STARTTLS");
         // TODO: check that this capability exists??
         // TODO: issue the STARTTLS command to the server
+        let resp = self.execute(Command::Starttls).await?;
+        dbg!(resp.wait().await?);
+        debug!("received OK from server");
 
         // issue exit to the read loop and retrieve the read half
         let _ = self.read_exit.send(());
@@ -188,11 +191,13 @@ where
     let exit = exit.fuse();
     pin_mut!(exit);
     loop {
+        debug!("READ LOOP ITER");
         let next = framed.next().fuse();
         pin_mut!(next);
 
         // only listen for a new command if there isn't one already
-        let mut cmd_fut = if let Some(_) = curr_cmd {
+        let mut cmd_fut = if let Some(ref cmd) = curr_cmd {
+            debug!("current command: {:?}", cmd);
             // if there is one, just make a future that never resolves so it'll always pick
             // the other options in the select.
             future::pending().boxed().fuse()
@@ -202,14 +207,15 @@ where
 
         select! {
             // read a command from the command list
-            mut command = cmd_fut => {
+            command = cmd_fut => {
                 if curr_cmd.is_none() {
-                    if let Some(CommandContainer { tag, command, .. }) = command.take() {
-                        let _ = write_tx.send(TaggedCommand(tag, command));
+                    if let Some(CommandContainer { ref tag, ref command, .. }) = command {
+                        let _ = write_tx.send(TaggedCommand(tag.clone(), command.clone()));
                         // let cmd_str = format!("{} {:?}\r\n", tag, cmd);
                         // write_tx.send(cmd_str);
                     }
                     curr_cmd = command;
+                    debug!("new command: {:?}", curr_cmd);
                 }
             }
 
@@ -232,6 +238,12 @@ where
                         let _ = channel.send(resp);
                         // debug!("res0: {:?}", res);
                     }
+                } else if let Response::Tagged(_, Condition { status: Status::Ok, ..}) = resp {
+                    // clear curr_cmd so another one can be sent
+                    if let Some(CommandContainer { channel, .. }) = curr_cmd.take() {
+                        let _ = channel.send(resp);
+                        // debug!("res0: {:?}", res);
+                    }
                 } else if let Some(CommandContainer { channel, .. }) = curr_cmd.as_mut() {
                     // we got a response from the server for this command, so send it over the
                     // channel
@@ -241,6 +253,7 @@ where
                     // debug!("res1: {:?}", res);
                 }
             }
+
             _ = exit => break,
         }
     }
@@ -257,8 +270,9 @@ where
     C: AsyncWrite,
 {
     // set up framed communication
-    let codec = ImapCodec::default();
-    let mut framed = FramedWrite::new(stream, codec);
+    // let codec = ImapCodec::default();
+    let mut stream = BufWriter::new(stream);
+    // let mut framed = FramedWrite::new(stream, codec);
 
     let mut exit_rx = exit_rx.map_err(|_| ()).shared();
     loop {
@@ -269,15 +283,20 @@ where
             command = command_fut => {
                 // TODO: handle errors here
                 if let Some(command) = command {
-                    let _ = framed.send(&command).await;
+                    let cmd = format_bytes!(b"{} {}\r\n", &*command.0.0, command.1);
+                    debug!("sending command: {:?}", String::from_utf8_lossy(&cmd));
+                    let _ = stream.write_all(&cmd).await;
+                    let _ = stream.flush().await;
+                    // let _ = framed.send(&command).await;
+                    // let _ = framed.flush().await;
                 }
                 // let _ = stream.write_all(line.as_bytes()).await;
                 // let _ = stream.flush().await;
-                // trace!("C>>>S: {:?}", line);
             }
             _ = exit_rx => break,
         }
     }
 
-    framed.into_inner()
+    // framed.into_inner()
+    stream.into_inner()
 }
