@@ -1,6 +1,8 @@
 macro_rules! rule {
     ($vis:vis $name:ident : $ret:ty => $expr:expr) => {
-        $vis fn $name(i: crate::proto::bytes::Bytes) -> nom::IResult<crate::proto::bytes::Bytes, $ret> {
+        $vis fn $name (
+            i: crate::proto::bytes::Bytes
+        ) -> crate::proto::parsers::VResult<crate::proto::bytes::Bytes, $ret> {
             $expr(i)
         }
     };
