@@ -8,6 +8,8 @@ mod macros;
 pub mod bytes;
 #[macro_use]
 pub mod parsers;
+pub mod convert_error;
+pub mod formatter;
 
 // data types
 pub mod command;
@@ -20,3 +22,7 @@ pub mod rfc6154;
 
 #[cfg(feature = "rfc2177")]
 pub mod rfc2177;
+
+// tests
+#[cfg(test)]
+pub mod test_rfc3501;
