@@ -1,12 +1,14 @@
 use anyhow::Result;
 use nom::{
-    error::{Error, ErrorKind, ParseError},
+    error::{ErrorKind, ParseError, VerboseError},
     CompareResult, Err, IResult, InputLength, Needed, Parser, ToUsize,
 };
 
 use super::bytes::{ShitCompare, ShitNeededForParsing};
 use super::rfc2234::is_digit;
 
+pub type VResult<I, O> = IResult<I, O, VerboseError<I>>;
+
 /// `sep_list!(t, d)` represents `t *(d t)` and automatically collapses it into
 /// `Vec<T>`.
 ///
@@ -79,7 +81,12 @@ pub fn parse_u32(s: impl AsRef<[u8]>) -> Result<u32> {
 }
 
 /// Always fails, used as a no-op.
-pub fn never<I, O>(i: I) -> IResult<I, O> { Err(Err::Error(Error::new(i, ErrorKind::Not))) }
+pub fn never<I, O, E>(i: I) -> IResult<I, O, E>
+where
+    E: ParseError<I>,
+{
+    Err(Err::Error(E::from_error_kind(i, ErrorKind::Not)))
+}
 
 /// Skip the part of the input matched by the given parser.
 pub fn skip<E, F, I, O>(mut f: F) -> impl FnMut(I) -> IResult<I, (), E>
