@@ -128,7 +128,7 @@ pub enum Capability {
 pub enum MailboxData {
     Flags(Vec<Flag>),
     List(MailboxList),
-    Lsub,
+    Lsub(MailboxList),
     Search(Vec<u32>),
     Status,
     Exists(u32),
