@@ -3,7 +3,11 @@
 //!
 //! Grammar from <https://tools.ietf.org/html/rfc2234#section-6.1>
 
-use nom::{branch::alt, multi::many0, sequence::pair};
+use nom::{
+    branch::alt,
+    multi::many0,
+    sequence::{pair, preceded},
+};
 
 use super::parsers::{byte, satisfy, skip};
 
@@ -35,7 +39,7 @@ rule!(pub HTAB : u8 => byte(b'\x09'));
 pub(crate) fn is_lf(c: u8) -> bool { c == b'\x0a' }
 rule!(pub LF : u8 => satisfy(is_lf));
 
-rule!(pub LWSP : () => skip(many0(alt((skip(WSP), skip(pair(CRLF, WSP)))))));
+rule!(pub LWSP : () => skip(many0(alt((WSP, preceded(CRLF, WSP))))));
 
 // rule!(pub OCTET : char => anychar);
 
