@@ -8,11 +8,10 @@ use nom::{
     combinator::{map, map_res, opt, verify},
     multi::{many0, many1},
     sequence::{delimited, pair, preceded, separated_pair, terminated, tuple},
-    IResult,
 };
 
 use super::bytes::Bytes;
-use super::parsers::{byte, never, parse_u32, satisfy, tagi, take, take_while1};
+use super::parsers::{byte, never, parse_u32, satisfy, tagi, take, take_while1, VResult};
 use super::response::{
     Address, Atom, Capability, Condition, Envelope, Flag, Mailbox, MailboxData, MailboxList,
     MailboxListFlag, MessageAttribute, Response, ResponseCode, ResponseText, Status, Tag,
@@ -118,6 +117,7 @@ rule!(pub flag : Flag => alt((
     map(tagi(b"\\Deleted"), |_| Flag::Deleted),
     map(tagi(b"\\Seen"), |_| Flag::Seen),
     map(tagi(b"\\Draft"), |_| Flag::Draft),
+    map(flag_keyword, Flag::Keyword),
     map(flag_extension, Flag::Extension),
 )));
 
@@ -125,6 +125,8 @@ rule!(pub flag_extension : Atom => preceded(byte(b'\\'), atom));
 
 rule!(pub flag_fetch : Flag => alt((flag, map(tagi(b"\\Recent"), |_| Flag::Recent))));
 
+rule!(pub flag_keyword : Atom => atom);
+
 rule!(pub flag_list : Vec<Flag> => paren!(sep_list!(?flag)));
 
 pub(crate) fn is_list_wildcards(c: u8) -> bool { c == b'%' || c == b'*' }
@@ -135,24 +137,13 @@ rule!(pub list_wildcards : u8 => satisfy(is_list_wildcards));
 // TODO: Future work, could possibly initialize writing to file if the length is
 // determined to exceed a certain threshold so we don't have insane amounts of
 // data in memory
-pub fn literal(i: Bytes) -> IResult<Bytes, Bytes> {
+pub fn literal(i: Bytes) -> VResult<Bytes, Bytes> {
     let mut length_of = terminated(delimited(byte(b'{'), number, byte(b'}')), CRLF);
     let (i, length) = length_of(i)?;
     debug!("length is: {:?}", length);
     map(take(length), Bytes::from)(i)
 }
 
-#[test]
-fn test_literal() {
-    assert_eq!(
-        literal(Bytes::from(b"{13}\r\nHello, world!"))
-            .unwrap()
-            .1
-            .as_ref(),
-        b"Hello, world!"
-    );
-}
-
 rule!(pub mailbox : Mailbox => alt((
     map(tagi(b"INBOX"), |_| Mailbox::Inbox),
     map(astring, Mailbox::Name),
@@ -161,6 +152,9 @@ rule!(pub mailbox : Mailbox => alt((
 rule!(pub mailbox_data : MailboxData => alt((
     map(preceded(pair(tagi(b"FLAGS"), SP), flag_list), MailboxData::Flags),
     map(preceded(pair(tagi(b"LIST"), SP), mailbox_list), MailboxData::List),
+    map(preceded(pair(tagi(b"LSUB"), SP), mailbox_list), MailboxData::Lsub),
+    map(terminated(number, pair(SP, tagi(b"EXISTS"))), MailboxData::Exists),
+    map(terminated(number, pair(SP, tagi(b"RECENT"))), MailboxData::Recent),
 )));
 
 rule!(pub mailbox_list : MailboxList => map(separated_pair(
@@ -217,7 +211,7 @@ rule!(pub nil : Bytes => tagi(b"NIL"));
 
 rule!(pub nstring : Option<Bytes> => opt_nil!(string));
 
-pub(crate) fn number(i: Bytes) -> IResult<Bytes, u32> {
+pub(crate) fn number(i: Bytes) -> VResult<Bytes, u32> {
     map_res(take_while1(is_digit), parse_u32)(i)
 }
 
